#include "mgos_barometer_bme280.h"
#include "mgos_i2c.h"

// Datasheet:
// https://cdn-shop.adafruit.com/datasheets/MPL115A2.pdf

bool mgos_barometer_bme280_detect(struct mgos_barometer *dev) {
  int val;

  if (!dev) return false;

  if ((val = mgos_i2c_read_reg_b(dev->i2c, dev->i2caddr, BME280_REG_DEVID)) < 0) {
    return false;
  }

  if (val == 0x56 || val == 0x57) {
    LOG(LL_INFO, ("Preproduction version of BMP280 detected (0x%02x)", val));
    return true;
  }
  if (val == 0x58) // Mass production BMP280
    return true;

  if (val == 0x60) // Mass production BME280
    return true;

  return true;
}

bool mgos_barometer_bme280_create(struct mgos_barometer *dev) {
  struct mgos_barometer_bme280_data *bme280_data;

  if (!dev) return false;

  bme280_data=calloc(1, sizeof(struct mgos_barometer_bme280_data));
  if (!bme280_data) return false;
  dev->user_data=bme280_data;

  if (!mgos_i2c_read_reg_n(dev->i2c, dev->i2caddr, BME280_REG_TEMPERATURE_CALIB_DIG_T1_LSB, 24, (uint8_t *)bme280_data))
    return false;

  // Mode | Pressure OS | Temp OS
  if (!mgos_i2c_write_reg_b(dev->i2c, dev->i2caddr, BME280_REG_CTRL_MEAS,  BME280_MODE_SLEEP | BME280_OVERSAMP_1X << 2 | BME280_OVERSAMP_1X << 5))
    return false;
  mgos_usleep(10000);

  // SPI | 250ms period | no filter
  if (!mgos_i2c_write_reg_b(dev->i2c, dev->i2caddr, BME280_REG_CONFIG,  0x00 | 0x04 << 2 | 0x00 << 5))
    return false;
  mgos_usleep(10000);

  // Mode | Pressure OS | Temp OS
  if (!mgos_i2c_write_reg_b(dev->i2c, dev->i2caddr, BME280_REG_CTRL_MEAS,  BME280_MODE_NORMAL | BME280_OVERSAMP_1X << 2 | BME280_OVERSAMP_1X << 5))
    return false;

  dev->has_barometer=true;
  dev->has_thermometer=true;

  return true;
}

bool mgos_barometer_bme280_destroy(struct mgos_barometer *dev) {
  if (!dev) return false;
  if (dev->user_data) {
    free(dev->user_data);
    dev->user_data=NULL;
  }
  return true;
}

bool mgos_barometer_bme280_read(struct mgos_barometer *dev) {
  struct mgos_barometer_bme280_data *bme280_data;

  if (!dev) return false;
  bme280_data=(struct mgos_barometer_bme280_data *) dev->user_data;
  if (!bme280_data) return false;

  // Forced Mode | 8x OS Pressure | 1x OS Temp
  // 0x01 | 0x04 << 2 | 0x01 << 5
  if (!mgos_i2c_write_reg_b(dev->i2c, dev->i2caddr, BME280_REG_CTRL_MEAS,  0x01 | 0x04 << 2 | 0x01 << 5))
    return false;
  mgos_usleep(10000);

  // read data from sensor
  uint8_t data[6];
  if (!mgos_i2c_read_reg_n(dev->i2c, dev->i2caddr, BME280_REG_PRESSURE_MSB, 6, data))
    return false;
  int32_t Padc, Tadc;
  Padc = (int32_t)((((uint32_t)(data[0])) << 12) | (((uint32_t)(data[1])) << 4) | ((uint32_t)data[2] >> 4));
  Tadc = (int32_t)((((uint32_t)(data[3])) << 12) | (((uint32_t)(data[4])) << 4) | ((uint32_t)data[5] >> 4));
//  LOG(LL_DEBUG, ("Padc=%d Tadc=%d", Padc, Tadc));

  int32_t var1, var2, T, t_fine, P;
  // Temp conversion:
  var1 = ((((Tadc >> 3) - ((int32_t)bme280_data->calib.dig_T1 << 1))) * ((int32_t)bme280_data->calib.dig_T2)) >> 11;
  var2  = (((((Tadc >> 4) - ((int32_t)bme280_data->calib.dig_T1)) * ((Tadc >> 4) - ((int32_t)bme280_data->calib.dig_T1))) >> 12) * ((int32_t)bme280_data->calib.dig_T3)) >> 14;
  t_fine = var1 + var2;
  T = (t_fine * 5 + 128) >> 8; // in hundredth of degrees C; 5123 means 51.23C
  dev->temperature=(float)T/100.0;

  // Pressure conversion:
  var1 = ((int64_t)t_fine) - 128000;
  var2 = var1 * var1 * (int64_t)bme280_data->calib.dig_P6;
  var2 = var2 + ((var1*(int64_t)bme280_data->calib.dig_P5) << 17);
  var2 = var2 + (((int64_t)bme280_data->calib.dig_P4) << 35);
  var1 = ((var1 * var1 * (int64_t)bme280_data->calib.dig_P3) >> 8) + ((var1 * (int64_t)bme280_data->calib.dig_P2) << 12);
  var1 = (((((int64_t)1) << 47) + var1)) * ((int64_t)bme280_data->calib.dig_P1) >> 33;
  if (var1 == 0) {
    P=0;
  } else {
    P = 1048576 - Padc;
    P = (((P << 31) - var2) * 3125) / var1;
    var1 = (((int64_t)bme280_data->calib.dig_P9) * (P >> 13) * (P >> 13)) >> 25;
    var2 = (((int64_t)bme280_data->calib.dig_P8) * P) >> 19;
    P = ((P + var1 + var2) >> 8) + (((int64_t)bme280_data->calib.dig_P7) << 4);
  }
  dev->pressure=((float)P/256.0)*1000;
//  LOG(LL_DEBUG, ("P=%.2f T=%.2f", dev->pressure, dev->temperature));

  return true;
}

